@@ -1,7 +1,8 @@
 from __future__ import print_function
+from stretch_body.robot_params import RobotParams
 import stretch_body.hello_utils as hello_utils
 import time
-
+import logging, logging.config
 
 
 class DeviceTimestamp:
@@ -21,35 +22,20 @@ class DeviceTimestamp:
         s=(self.timestamp_base + ts - self.timestamp_first) / 1000000.0
         return self.ts_start+s
 
+
 class Device:
+    logging_params = RobotParams.get_params()[1]['logging']
+    logging.config.dictConfig(logging_params)
     """
     Generic base class for all custom Stretch hardware
     """
-    def __init__(self,verbose=False):
-        #Factory + Tool params form the robot_params
-        #User params can overwrite the resulting robot_params
-        self.verbose=verbose
-        self.user_params=hello_utils.read_fleet_yaml('stretch_re1_user_params.yaml')
-        self.robot_params=hello_utils.read_fleet_yaml(self.user_params['factory_params'])
-        self.robot_params.update(hello_utils.read_fleet_yaml(self.user_params['tool_params']))
-        self.overwrite_params(self.robot_params,self.user_params)
+    def __init__(self, name=''):
+        self.name = name
+        self.user_params, self.robot_params = RobotParams.get_params()
+        self.params = self.robot_params.get(self.name, {})
+        self.logger = logging.getLogger(self.name)
         self.timestamp = DeviceTimestamp()
 
-
-    def overwrite_params(self,factory_dict,user_dict):
-        for k in user_dict.keys():
-            if k in factory_dict:
-                if type(factory_dict[k])==type(user_dict[k]):
-                    if type(factory_dict[k])==dict:
-                        self.overwrite_params(factory_dict[k],user_dict[k])
-                    else:
-                        factory_dict[k]=user_dict[k]
-                else:
-                    print('Overwritting Factory Params with User Params. Type mismatch for key:',k)
-            else: #If key not present, add anyhow (useful for adding new end_of_arm)
-                factory_dict[k] = user_dict[k]
-
-
     # ########### Primary interface #############
 
     def startup(self):
@@ -60,15 +46,20 @@ class Device:
 
     def push_command(self):
         pass
+
     def pull_status(self):
         pass
 
     def home(self):
         pass
 
-    def pretty_print(self):
+    def step_sentry(self,robot):
         pass
 
+    def pretty_print(self):
+        print('----- {0} ------ '.format(self.name))
+        hello_utils.pretty_print_dict("params", self.params)
+
     def write_device_params(self,device_name, params):
         rp=hello_utils.read_fleet_yaml(self.user_params['factory_params'])
         rp[device_name]=params
