@@ -10,16 +10,13 @@ class EndOfArm(DynamixelXChain):
     simply deriving it from DynamixelHelloXL430 and declaring the class name / Python module name
     in the User YAML file
     """
-    def __init__(self,verbose=False):
-        DynamixelXChain.__init__(self,'/dev/hello-dynamixel-wrist',verbose)
-        self.name='end_of_arm'
-        self.params=self.robot_params[self.name]
-        self.joints=self.params['devices'].keys()
+    def __init__(self, name='end_of_arm'):
+        DynamixelXChain.__init__(self, usb='/dev/hello-dynamixel-wrist', name=name)
+        self.joints = self.params.get('devices', {}).keys()
         for j in self.joints:
-            module_name=self.params['devices'][j]['py_module_name']
-            class_name=self.params['devices'][j]['py_class_name']
-            dynamixel_device=getattr(importlib.import_module(module_name), class_name)(self)
-            dynamixel_device.verbose=verbose
+            module_name = self.params['devices'][j]['py_module_name']
+            class_name = self.params['devices'][j]['py_class_name']
+            dynamixel_device = getattr(importlib.import_module(module_name), class_name)(chain=self)
             self.add_motor(dynamixel_device)
 
     def move_to(self, joint,x_r, v_r=None, a_r=None):
@@ -52,12 +49,21 @@ class EndOfArm(DynamixelXChain):
         with self.pt_lock:
             self.motors[joint].pose(p, v_r, a_r)
 
-    def home(self, joint):
+    def stow(self):
+        print('--------- Stowing Wrist Yaw ----')
+        self.move_to('wrist_yaw', self.params['stow']['wrist_yaw'])
+
+    def home(self, joint=None):
         """
         Home to hardstops
         """
-        with self.pt_lock:
-            self.motors[joint].home()
+        if joint is None:
+            for j in self.joints:
+                print('--------- Homing %s ----'%j)
+                self.motors[j].home()
+        else:
+            with self.pt_lock:
+                self.motors[joint].home()
 
     def is_tool_present(self,class_name):
         """
