@@ -7,12 +7,11 @@ class Lift(Device):
     """
     API to the Stretch RE1 Lift
     """
-    def __init__(self,verbose=False):
-        Device.__init__(self,verbose)
-        self.name='lift'
-        self.params=self.robot_params[self.name]
-        self.motor = Stepper('/dev/hello-motor-lift',verbose=verbose)
+    def __init__(self):
+        Device.__init__(self, 'lift')
+        self.motor = Stepper(usb='/dev/hello-motor-lift')
         self.status = {'timestamp_pc':0,'pos': 0.0, 'vel': 0.0, 'force':0.0,'motor': self.motor.status}
+
         # Default controller params
         self.stiffness = 1.0
         self.i_feedforward=self.params['i_feedforward']
@@ -21,6 +20,7 @@ class Lift(Device):
         self.i_contact_neg = self.translate_force_to_motor_current(self.params['contact_thresh_N'][0])
         self.i_contact_pos = self.translate_force_to_motor_current(self.params['contact_thresh_N'][1])
         self.motor.set_motion_limits(self.translate_to_motor_rad(self.params['range_m'][0]), self.translate_to_motor_rad(self.params['range_m'][1]))
+        self.soft_motion_limits = [self.params['range_m'][0], self.params['range_m'][1]]
     # ###########  Device Methods #############
 
     def startup(self):
@@ -48,6 +48,13 @@ class Lift(Device):
         #self.motor.pretty_print()
 
     # ###################################################
+    def set_soft_motion_limits(self,x_min=None,x_max=None):
+        x_min = max(x_min,self.params['range_m'][0]) if x_min is not None else self.params['range_m'][0]
+        x_max = min(x_max,self.params['range_m'][1]) if x_max is not None else self.params['range_m'][1]
+        if x_min != self.soft_motion_limits[0] or x_max != self.soft_motion_limits[1]:
+            self.motor.set_motion_limits(self.translate_to_motor_rad(x_min), self.translate_to_motor_rad(x_max))
+
+        self.soft_motion_limits=[x_min, x_max]
 
     def move_to(self,x_m,v_m=None, a_m=None, stiffness=None, contact_thresh_pos_N=None, contact_thresh_neg_N=None, req_calibration=True):
         """
@@ -61,9 +68,9 @@ class Lift(Device):
         """
         if req_calibration:
             if not self.motor.status['pos_calibrated']:
-                print('Lift not calibrated')
+                self.logger.warn('Lift not calibrated')
                 return
-
+            x_m = min(max(self.soft_motion_limits[0], x_m), self.soft_motion_limits[1]) #Only clip motion when calibrated
         if stiffness is not None:
             stiffness = max(0, min(1.0, stiffness))
         else:
@@ -112,9 +119,14 @@ class Lift(Device):
         """
         if req_calibration:
             if not self.motor.status['pos_calibrated']:
-                print('Lift not calibrated')
+                self.logger.warn('Lift not calibrated')
                 return
 
+            if self.status['pos'] + x_m < self.soft_motion_limits[0]:  #Only clip motion when calibrated
+                x_m = self.soft_motion_limits[0] - self.status['pos']
+            if self.status['pos'] + x_m > self.soft_motion_limits[1]:
+                x_m = self.soft_motion_limits[1] - self.status['pos']
+
         if stiffness is not None:
             stiffness = max(0, min(1.0, stiffness))
         else:
@@ -190,7 +202,7 @@ class Lift(Device):
 
     def home(self, measuring=False):
         if not self.motor.hw_valid:
-            print('Not able to home lift. Hardware not present')
+            self.logger.warn('Not able to home lift. Hardware not present')
             return
         print('Homing lift...')
         self.motor.enable_guarded_mode()
@@ -222,7 +234,7 @@ class Lift(Device):
         time.sleep(1.0)
 
         # Down direction
-        self.move_by(x_m=-0.5, req_calibration=False)
+        self.move_to(x_m=0.6, req_calibration=False)
         self.push_command()
         time.sleep(6.0)
 
@@ -234,3 +246,6 @@ class Lift(Device):
         self.push_command()
 
         return x_up
+
+    def step_sentry(self,robot):
+        self.motor.step_sentry(robot)
