@@ -2,7 +2,6 @@ from __future__ import print_function
 import threading
 import time
 import signal
-import logging
 import os
 import importlib
 
@@ -19,7 +18,8 @@ import stretch_body.hello_utils as hello_utils
 from serial import SerialException
 
 from stretch_body.robot_monitor import RobotMonitor
-from stretch_body.robot_sentry import RobotSentry
+from stretch_body.robot_collision import RobotCollision
+
 
 # #############################################################
 class RobotDynamixelThread(threading.Thread):
@@ -44,7 +44,7 @@ class RobotDynamixelThread(threading.Thread):
             tsleep = max(0.001, (1 / self.robot_update_rate_hz) - (te - ts))
             if not self.shutdown_flag.is_set():
                 time.sleep(tsleep)
-
+        print('Shutting down RobotDynamixelThread')
 
 
 class RobotThread(threading.Thread):
@@ -59,11 +59,12 @@ class RobotThread(threading.Thread):
 
         self.robot_update_rate_hz = 25.0  #Hz
         self.monitor_downrate_int = 5  # Step the monitor at every Nth iteration
+        self.collision_downrate_int = 5  # Step the monitor at every Nth iteration
         self.sentry_downrate_int = 2  # Step the sentry at every Nth iteration
         if self.robot.params['use_monitor']:
             self.robot.monitor.startup()
-        if self.robot.params['use_sentry']:
-            self.robot.sentry.startup()
+        if self.robot.params['use_collision_manager']:
+            self.robot.collision.startup()
         self.shutdown_flag = threading.Event()
         self.timer_stats = hello_utils.TimerStats()
         self.titr=0
@@ -74,49 +75,51 @@ class RobotThread(threading.Thread):
             ts = time.time()
             self.robot._pull_status_non_dynamixel()
             self.first_status = True
+
             if self.robot.params['use_monitor']:
                 if (self.titr % self.monitor_downrate_int) == 0:
                     self.robot.monitor.step()
 
+            if self.robot.params['use_collision_manager']:
+                    self.robot.collision.step()
+
             if self.robot.params['use_sentry']:
                 if (self.titr % self.sentry_downrate_int) == 0:
-                    self.robot.sentry.step()
+                    self.robot._step_sentry()
 
             self.titr=self.titr+1
             te = time.time()
             tsleep = max(0.001, (1 / self.robot_update_rate_hz) - (te - ts))
             if not self.shutdown_flag.is_set():
                 time.sleep(tsleep)
+        print('Shutting down RobotThread')
 
 
 class Robot(Device):
     """
     API to the Stretch RE1 Robot
     """
-    def __init__(self,verbose=False):
-        Device.__init__(self,verbose)
-        self.params=self.robot_params['robot']
-        self.monitor = RobotMonitor(self,verbose=verbose)
-        self.sentry = RobotSentry(self,verbose=verbose)
+    def __init__(self):
+        Device.__init__(self, 'robot')
+        self.monitor = RobotMonitor(self)
+        self.collision = RobotCollision(self)
         self.dirty_push_command = False
         self.lock = threading.RLock() #Prevent status thread from triggering motor sync prematurely
-
         self.status = {'pimu': {}, 'base': {}, 'lift': {}, 'arm': {}, 'head': {}, 'wacc': {}, 'end_of_arm': {}}
 
-
-        self.pimu=pimu.Pimu(verbose=verbose)
+        self.pimu=pimu.Pimu()
         self.status['pimu']=self.pimu.status
 
-        self.base=base.Base(verbose=verbose)
+        self.base=base.Base()
         self.status['base']=self.base.status
 
-        self.lift=lift.Lift(verbose=verbose)
+        self.lift=lift.Lift()
         self.status['lift']=self.lift.status
 
-        self.arm=arm.Arm(verbose=verbose)
+        self.arm=arm.Arm()
         self.status['arm']=self.arm.status
 
-        self.head=head.Head(verbose=verbose)
+        self.head=head.Head()
         self.status['head']=self.head.status
 
         if 'custom_wacc' in self.params:
@@ -124,17 +127,20 @@ class Robot(Device):
             class_name = self.params['custom_wacc']['py_class_name']
             self.wacc=getattr(importlib.import_module(module_name), class_name)(self)
         else:
-            self.wacc=wacc.Wacc(verbose=verbose)
+            self.wacc=wacc.Wacc()
         self.status['wacc']=self.wacc.status
 
-        self.end_of_arm=end_of_arm.EndOfArm(verbose=verbose)
-        self.status['end_of_arm']=self.end_of_arm.status
+
+        tool_name = self.params['tool']
+        module_name = self.robot_params[tool_name]['py_module_name']
+        class_name = self.robot_params[tool_name]['py_class_name']
+        self.end_of_arm = getattr(importlib.import_module(module_name), class_name)()
+        self.status['end_of_arm'] = self.end_of_arm.status
 
         self.devices={ 'pimu':self.pimu, 'base':self.base, 'lift':self.lift, 'arm': self.arm, 'head': self.head, 'wacc':self.wacc, 'end_of_arm':self.end_of_arm}
         self.rt=None
         self.dt=None
 
-
     # ###########  Device Methods #############
 
     def startup(self):
@@ -142,28 +148,7 @@ class Robot(Device):
         To be called once after class instantiation.
         Prepares devices for communications and motion
         """
-
-        # Set up logging
-        t = time.localtime()
-        capture_date = str(t.tm_year) + str(t.tm_mon).zfill(2) + str(t.tm_mday).zfill(2) + str(t.tm_hour).zfill(
-            2) + str(t.tm_min).zfill(2)
-        self.log_filename = os.environ['HELLO_FLEET_PATH'] + '/log/' + self.params[
-            'serial_no'] + '_monitor_' + capture_date + '.log'
-        self.logger = logging.getLogger('robot')
-        self.logger.setLevel(logging.DEBUG)
-        fh = logging.FileHandler(self.log_filename)
-        fh.setLevel(logging.DEBUG)
-        formatter = logging.Formatter(
-            '%(asctime)s - ' + hello_utils.get_fleet_id() + ' - %(name)s - %(levelname)s - %(message)s')
-        fh.setFormatter(formatter)
-        self.logger.addHandler(fh)
-        if self.params['log_to_console']:
-            ch = logging.StreamHandler()
-            ch.setLevel(logging.DEBUG)
-            ch.setFormatter(formatter)
-            self.logger.addHandler(ch)
-
-        self.logger.info('Starting up Robot')
+        self.logger.debug('Starting up Robot {0} of batch {1}'.format(self.params['serial_no'], self.params['batch_name']))
         for k in self.devices.keys():
             if self.devices[k] is not None:
                 if not self.devices[k].startup():
@@ -175,13 +160,16 @@ class Robot(Device):
         # Register the signal handlers
         signal.signal(signal.SIGTERM, hello_utils.thread_service_shutdown)
         signal.signal(signal.SIGINT, hello_utils.thread_service_shutdown)
+
         self.rt = RobotThread(self)
         self.rt.setDaemon(True)
         self.rt.start()
+
         self.dt = RobotDynamixelThread(self)
         self.dt.setDaemon(True)
         self.dt.start()
-        #Wait for threads to start reading data
+
+        # Wait for status reading threads to start reading data
         ts=time.time()
         while not self.rt.first_status and not self.dt.first_status and time.time()-ts<3.0:
            time.sleep(0.1)
@@ -193,18 +181,18 @@ class Robot(Device):
         To be called once before exiting a program
         Cleanly stops down motion and communication
         """
-        print('Shutting down robot...')
+        print('---- Shutting down robot ----')
         if self.rt is not None:
             self.rt.shutdown_flag.set()
-            self.rt.join()
+            self.rt.join(1)
         if self.dt is not None:
             self.dt.shutdown_flag.set()
-            self.dt.join()
+            self.dt.join(1)
         for k in self.devices.keys():
             if self.devices[k] is not None:
                 print('Shutting down',k)
                 self.devices[k].stop()
-
+        print('---- Shutdown complete ----')
 
     def get_status(self):
         """
@@ -220,7 +208,7 @@ class Robot(Device):
         print('Time',time.time())
         print('Serial No',self.params['serial_no'])
         print('Batch', self.params['batch_name'])
-        self._pretty_print_dict('Status',s)
+        hello_utils.pretty_print_dict('Status',s)
 
 
     def push_command(self):
@@ -257,9 +245,13 @@ class Robot(Device):
         self.head.move_to('head_tilt',self.params['stow']['head_tilt'])
 
         lift_stowed=False
+        pos_lift = self.params['stow']['lift']
+        if 'stow' in self.end_of_arm.params:  # Allow tool defined stow position to take precedence
+            if 'lift' in self.end_of_arm.params['stow']:
+                pos_lift = self.end_of_arm.params['stow']['lift']
         if self.lift.status['pos']<=self.params['stow']['lift']: #Needs to come up before bring in arm
             print('--------- Stowing Lift ----')
-            self.lift.move_to(self.params['stow']['lift'])
+            self.lift.move_to(pos_lift)
             self.push_command()
             time.sleep(0.25)
             ts = time.time()
@@ -269,25 +261,26 @@ class Robot(Device):
 
         #Bring in arm before bring down
         print('--------- Stowing Arm ----')
-        self.arm.move_to(self.params['stow']['arm'])
+        pos_arm = self.params['stow']['arm']
+        if 'stow' in self.end_of_arm.params:  # Allow tool defined stow position to take precedence
+            if 'arm' in self.end_of_arm.params['stow']:
+                pos_arm = self.end_of_arm.params['stow']['arm']
+
+        self.arm.move_to(pos_arm)
         self.push_command()
         time.sleep(0.25)
         ts = time.time()
         while not self.arm.motor.status['near_pos_setpoint'] and time.time() - ts < 3.0:
             time.sleep(0.1)
 
-        # Fold in wrist and gripper
-        print('--------- Stowing Wrist Yaw ----')
-        self.end_of_arm.move_to('wrist_yaw', self.params['stow']['wrist_yaw'])
-        if self.end_of_arm.is_tool_present('StretchGripper'):
-            self.end_of_arm.move_to('stretch_gripper', self.params['stow']['stretch_gripper'])
+        self.end_of_arm.stow()
         time.sleep(0.25)
 
 
         #Now bring lift down
         if not lift_stowed:
             print('--------- Stowing Lift ----')
-            self.lift.move_to(self.params['stow']['lift'])
+            self.lift.move_to(pos_lift)
             self.push_command()
             time.sleep(0.25)
             ts = time.time()
@@ -319,23 +312,12 @@ class Robot(Device):
 
         # Home the end-of-arm
         if self.end_of_arm is not None:
-            for j in self.end_of_arm.joints:
-                print( '--------- Homing ', j, '----')
-                self.end_of_arm.home(j)
+            self.end_of_arm.home()
         #Let user know it is done
         self.pimu.trigger_beep()
         self.push_command()
     # ################ Helpers #################################
 
-    def _pretty_print_dict(self, t, d):
-        print('--------', t, '--------')
-        for k in d.keys():
-            if type(d[k]) != dict:
-                print(k, ' : ', d[k])
-        for k in d.keys():
-            if type(d[k]) == dict:
-                self._pretty_print_dict(k, d[k])
-
     def _pull_status_dynamixel(self):
         try:
             self.end_of_arm.pull_status()
@@ -349,3 +331,10 @@ class Robot(Device):
         self.lift.pull_status()
         self.arm.pull_status()
         self.pimu.pull_status()
+
+    def _step_sentry(self):
+        self.head.step_sentry(self)
+        self.base.step_sentry(self)
+        self.arm.step_sentry(self)
+        self.lift.step_sentry(self)
+        self.end_of_arm.step_sentry(self)
