@@ -5,7 +5,8 @@ import struct
 import array as arr
 import stretch_body.cobbs_framing as cobbs_framing
 import copy
-
+import fcntl
+import logging
 
 """
 
@@ -49,21 +50,20 @@ RPC_ACK_GET_BLOCK_LAST = 108
 RPC_BLOCK_SIZE = 32
 RPC_DATA_SIZE = 1024
 
-dbg_on = 1
+dbg_on = 0
 
 
 class Transport():
     """
     Handle serial communication with Devices
     """
-    def __init__(self, usb,verbose=False):
-        self.usb=usb
-        self.verbose=verbose
+    def __init__(self, usb, logger=logging.getLogger()):
+        self.usb = usb
+        self.logger = logger
         self.payload_out = arr.array('B', [0] * (RPC_DATA_SIZE+1))
         self.payload_in = arr.array('B', [0] * (RPC_DATA_SIZE+1))
         self.buf = arr.array('B', [0] *(RPC_BLOCK_SIZE*2))
 
-
         self.write_error=0
         self.read_error = 0
         self.rpc_queue=[]
@@ -71,14 +71,21 @@ class Transport():
         self.itr = 0
         self.itr_time = 0
         self.tlast = 0
-        if self.verbose:
-            print('Starting TransportConnection on: ' + self.usb)
+        self.logger.debug('Starting TransportConnection on: ' + self.usb)
         try:
-            self.ser = serial.Serial(self.usb)#PosixPollSerial(self.usb)#Serial(self.usb)# 115200)  # , write_timeout=1.0)  # Baud not important since USB comms
+            self.ser = serial.Serial(self.usb, write_timeout=1.0)#PosixPollSerial(self.usb)#Serial(self.usb)# 115200)  # , write_timeout=1.0)  # Baud not important since USB comms
+            if self.ser.isOpen():
+                try:
+                    fcntl.flock(self.ser.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
+                except IOError:
+                    self.logger.error('Port %s is busy. Check if another Stretch Body process is already running'%self.usb)
+                    self.ser.close()
+                    self.ser=None
         except serial.SerialException as e:
-            print("SerialException({0}): {1}".format(e.errno, e.strerror))
+            self.logger.error("SerialException({0}): {1}".format(e.errno, e.strerror))
             self.ser = None
-
+        if self.ser==None:
+            self.logger.warn('Unable to open serial port for device %s'%self.usb)
         self.framer=cobbs_framing.CobbsFraming( )
         self.status={'rate':0,'read_error':0,'write_error':0,'itr':0,'transaction_time_avg':0,'transaction_time_max':0,'timestamp_pc':0}
 
@@ -88,8 +95,7 @@ class Transport():
 
     def stop(self):
         if self.ser:
-            if self.verbose:
-                print('Shutting down TransportConnection on: ' + self.usb)
+            self.logger.debug('Shutting down TransportConnection on: ' + self.usb)
             self.ser.close()
             self.ser = None
 
@@ -103,8 +109,7 @@ class Transport():
 
     def step_rpc(self,rpc,rpc_callback): #Handle a single RPC transaction
         if not self.ser:
-            if self.verbose:
-                print('Transport Serial not present for:',self.usb)
+            self.logger.debug('Transport Serial not present for: %s' % self.usb)
             return
 
         dbg_buf = ''
@@ -125,7 +130,7 @@ class Transport():
                     dbg_buf = dbg_buf +'Framer rcvd 0 bytes on RPC_ACK_NEW_RPC'
 
             if crc!=1 or self.buf[0] != RPC_ACK_NEW_RPC:
-                print('Transport RX Error on RPC_ACK_NEW_RPC', crc, nr, self.buf[0])
+                self.logger.error('Transport RX Error on RPC_ACK_NEW_RPC {0} {1} {2}'.format(crc, nr, self.buf[0]))
                 raise TransportError
             #if dbg_on:
             #    print('New RPC initiated, len',len(rpc))
@@ -154,7 +159,7 @@ class Transport():
                     #if dbg_on:
                     #    print('Last block ack rcvd',ntx)
                     if crc!=1 or self.buf[0]!=RPC_ACK_SEND_BLOCK_LAST:
-                        print('Transport RX Error on RPC_ACK_SEND_BLOCK_LAST', crc, nr, self.buf[0])
+                        self.logger.error('Transport RX Error on RPC_ACK_SEND_BLOCK_LAST {0} {1} {2}'.format(crc, nr, self.buf[0]))
                         raise TransportError
                 else:
                     self.buf[0] = RPC_SEND_BLOCK_MORE
@@ -173,7 +178,7 @@ class Transport():
                         else:
                             dbg_buf = dbg_buf + 'Framer rcvd 0 bytes on RPC_SEND_BLOCK_MORE'
                     if crc!=1 or self.buf[0]!=RPC_ACK_SEND_BLOCK_MORE:
-                        print('Transport RX Error on RPC_ACK_SEND_BLOCK_MORE', crc, nr, self.buf[0])
+                        self.logger.error('Transport RX Error on RPC_ACK_SEND_BLOCK_MORE {0} {1} {2}'.format(crc, nr, self.buf[0]))
                         raise TransportError
             ########### Receive all blocks
             reply = arr.array('B')
@@ -188,7 +193,7 @@ class Transport():
                 #if dbg_on:
                 #    print('Block request success')
                 if crc != 1 or not (self.buf[0] == RPC_ACK_GET_BLOCK_MORE or self.buf[0] == RPC_ACK_GET_BLOCK_LAST):
-                    print('Transport RX Error on RPC_GET_BLOCK', crc, nr, self.buf[0])
+                    self.logger.error('Transport RX Error on RPC_GET_BLOCK {0} {1} {2}'.format(crc, nr, self.buf[0]))
                     raise TransportError
                 reply=reply+self.buf[1:nr]
 
@@ -203,13 +208,20 @@ class Transport():
             if dbg_on:
                 print('---- Debug Exception')
                 print(dbg_buf)
-            print("Exception: TransportError")
             self.read_error = self.read_error + 1
             self.ser.reset_output_buffer()
             self.ser.reset_input_buffer()
-            #exit()
-
-
+            self.logger.error("TransportError: %s : %s" % (self.usb, str(e)))
+        except serial.SerialTimeoutException as e:
+            self.write_error += 1
+            self.ser=None
+            self.logger.error("SerialTimeoutException: %s : %s"%(self.usb, str(e)))
+        except serial.SerialException as e:
+            self.logger.error("SerialException: %s : %s"%(self.usb, str(e)))
+            self.ser=None
+        except TypeError as e:
+            self.logger.error("TypeError: %s : %s" % (self.usb, str(e)))
+            self.ser=None
 
     def is_step_complete(self):
         return self.rt.dirty_step==False
@@ -225,25 +237,17 @@ class Transport():
             self.ser.reset_input_buffer()
 
         #This will block until all RPCs have been commpleted
-        try:
-            #called by body thread at cyclic rate
-            self.itr += 1
-            self.itr_time = time.time() - self.tlast
-            self.tlast = time.time()
-            #Now run RPC calls
-            while len(self.rpc_queue):
-                rpc,reply_callback=self.rpc_queue[0]
-                #if dbg_on:
-                #    print('RPC of',reply_callback)
-                self.step_rpc(rpc,reply_callback)
-                self.rpc_queue = self.rpc_queue[1:]
-
-        except IOError as e:
-            print("IOError({0}): {1}".format(e.errno, e.strerror))
-            self.read_error=self.read_error+1
-        except serial.SerialTimeoutException as e:
-            self.write_error += 1
-            print("SerialException({0}): {1}".format(e.errno, e.strerror))
+        #called by body thread at cyclic rate
+        self.itr += 1
+        self.itr_time = time.time() - self.tlast
+        self.tlast = time.time()
+        #Now run RPC calls
+        while len(self.rpc_queue):
+            rpc,reply_callback=self.rpc_queue[0]
+            #if dbg_on:
+            #    print('RPC of',reply_callback)
+            self.step_rpc(rpc,reply_callback)
+            self.rpc_queue = self.rpc_queue[1:]
 
         # Update status
         if self.itr_time != 0:
@@ -261,19 +265,10 @@ class Transport():
             time.sleep(0.1)  # May have been a hard exit, give time for bad data to land, remove, do final RPC
             self.ser.reset_output_buffer()
             self.ser.reset_input_buffer()
-
-        #This will block until all RPCs have been commpleted
-        try:
-            while len(self.rpc_queue2):
-                rpc,reply_callback=self.rpc_queue2[0]
-                self.step_rpc(rpc,reply_callback)
-                self.rpc_queue2 = self.rpc_queue2[1:]
-        except IOError as e:
-            print("IOError({0}): {1}".format(e.errno, e.strerror))
-            self.read_error=self.read_error+1
-        except serial.SerialTimeoutException as e:
-            self.write_error += 1
-            print("SerialException({0}): {1}".format(e.errno, e.strerror))
+        while len(self.rpc_queue2):
+            rpc,reply_callback=self.rpc_queue2[0]
+            self.step_rpc(rpc,reply_callback)
+            self.rpc_queue2 = self.rpc_queue2[1:]
 
 
 # #####################################
@@ -282,7 +277,7 @@ def pack_string_t(s,sidx,x):
     return struct.pack_into(str(n)+'s',s,sidx,x)
 
 def unpack_string_t(s,n):
-    return (struct.unpack(str(n)+'s', s[:n])[0]).strip('\x00')
+    return (struct.unpack(str(n)+'s', s[:n])[0].strip(b'\x00')).decode('utf-8')
 
 def unpack_int32_t(s):
     return struct.unpack('i',s[:4])[0]
